'''
Created on Aug 26, 2013

@author: Patrick Dell'Era
'''
##############################################################################
### NZBGET POST-PROCESSING SCRIPT                                          ###

# Accurately finish dated episode renaming.
#
# This script uses Filebot to accurately rename a dated episode
# of the specified shows. The show names are finished with
# either the Broadcast Date details or the Season and Episode
# details.
#
# Filebot is called to finish the renaming as specified. See
# http://www.filebot.net/ for details.
#
# NOTE: This script requires Python 2 and Filebot to be installed on your system.

##############################################################################
### OPTIONS																   ###

# Series to finish with Season and Episode.
#
# The shows in these series will be finished with Series and Episode
# format (S01E01). The entries are separated with commas.
#
# Example=Real Time with Bill Maher, The Late Late Show
#SeasonEpisodeFormat=Real Time with Bill Maher

# Mask to finish the Season and Episode shows.
#
# This is a string that will be used for finishing dated episodes
# with season and episode details. The mask is in syntax understood
# by Filebot.
#
# Example={n.upperInitial().space('.')}.{s00e00}-{t}
#SeasonEpisodeMask={n.upperInitial().space('.')}.{s00e00}-{t}

# Series to finish with Broadcast Date.
#
# The shows in these series will be finished with date of the broadcast.
# The entries are separated with commas.
#
# Example=The Daily Show, The Colbert Report
#BroadcastDateFormat=The Daily Show, The Colbert Report

# Mask to finish the Broadcst Date shows.
#
# This is a string that will be used for finishing dated episodes
# with Broadcast Date details. The mask is in syntax understood
# by Filebot.
#
# Example={n.upperInitial().space('.')}.{airdate.format('yyy.MM.dd')}-{t}
#BroadcastDateMask={n.upperInitial().space('.')}.{airdate.format('yyy.MM.dd')}-{t}

# File extensions for video files.
#
# Only files with these extensions are processed. Extensions must
# be separated with commas. Spaces and dots may optionally be added
# for readability.
#
# Example=.mkv, .avi, .divx, .xvid, .mov, .wmv, .mp4, .mpg, .mpeg, .vob, .iso, .img, .ts
#VideoExtensions=.mkv, .avi, .divx, .xvid, .mov, .wmv, .mp4, .mpg, .mpeg, .vob, .iso, .img, .ts

### NZBGET POST-PROCESSING SCRIPT                                          ###
##############################################################################
import sys
from os.path import dirname
sys.path.append(dirname(__file__) + './videosort/lib')

import re
import glob
import urllib
import shutil
import os.path
import xml.dom.minidom
from subprocess import call
import childEnVars

DEBUG=False

# Exit codes used by NZBGet
POSTPROCESS_SUCCESS=93
POSTPROCESS_NONE=95
POSTPROCESS_ERROR=94

def process(strFilename, strRenameMask):
	# "filebot -rename -non-strict --db tvrage %fname% --format \"{n.space('.')}.{s00e00}-{t}\""
	strCmd = "filebot -rename -non-strict --db tvrage \"%s\" --format \"%s\"" % (strFilename, strRenameMask)
	try:
		retcode = call(strCmd, shell=True)
		if retcode < 0:
			print "[ERROR] Child was terminated by signal", -retcode
		#else:
			#print "[ERROR] Child returned", retcode
		return retcode
	except OSError as e:
		print "[ERROR] Execution failed:", e
	return 1

def extension_regex_pattern():
	video_extensions=[x.strip(' .') for x in os.environ.get('NZBPO_VIDEOEXTENSIONS', '.mkv,.avi,.divx,.xvid,.mov,.wmv,.mp4,.mpg,.mpeg,.vob,.iso,.img,.ts').split(',')]
	repattern = "("
	for i in range(len(video_extensions)):
		if i == len(video_extensions) - 1:
			repattern += r"\.%s)$" % video_extensions[i]
		else:
			repattern += r"\.%s|" % video_extensions[i]
	if DEBUG:
		print "video extensions regex=\"%s\"" % repattern
	return repattern

if __name__ == '__main__':
	# Check if the script is called from nzbget 11.0 or later
	if not 'NZBOP_SCRIPTDIR' in os.environ:
		print('*** NZBGet post-processing script ***')
		print('This script is supposed to be called from nzbget (11.0 or later).')
		sys.exit(POSTPROCESS_ERROR)

	# Check if directory still exist (for post-process again)
	if not os.path.exists(os.environ['NZBPP_DIRECTORY']) and not os.path.exists(childEnVars.Get('NZBPP_DIRECTORY')):
		print('[INFO] Destination directory doesn\'t exist, exiting')
		sys.exit(POSTPROCESS_NONE)

	# Check par and unpack status for errors
	if os.environ['NZBPP_PARSTATUS'] == '1' or os.environ['NZBPP_PARSTATUS'] == '4' or os.environ['NZBPP_UNPACKSTATUS'] == '1':
		print('[WARNING] Download of "%s" has failed, exiting' % (os.environ['NZBPP_NZBNAME']))
		sys.exit(POSTPROCESS_NONE)

	success = False
	finaldirectory = os.environ['NZBPP_DIRECTORY']
	if not os.path.exists(finaldirectory):
		finaldirectory = childEnVars.Get('NZBPP_DIRECTORY')
		originalname = childEnVars.Get('NZBPP_NZBNAME')
	else:
		originalname = os.environ['NZBPP_NZBNAME']

	if DEBUG:
		print "finaldirectory = \"%s\"" % finaldirectory
		print "originalname = \"%s\"" % originalname

	try:
		season_episode_shows=[x.strip().replace(' ', '.') for x in os.environ.get('NZBPO_SEASONEPISODEFORMAT', '').split(',')]
		broadcast_date_shows=[x.strip().replace(' ', '.') for x in os.environ.get('NZBPO_BROADCASTDATEFORMAT', '').split(',')]
		if DEBUG:
			print "se shows"
			print season_episode_shows
			print "bd shows"
			print broadcast_date_shows

		extparser = re.compile(extension_regex_pattern(), re.IGNORECASE)
		show_se_parsers = []
		show_date_parsers = []
		for show in season_episode_shows:
			show_se_parsers.append(re.compile(show, re.IGNORECASE))
		for show in broadcast_date_shows:
			show_date_parsers.append(re.compile(show, re.IGNORECASE))
	except Exception as e:
		print('[ERROR] Failed: Building regex parser expressions.')
		print('[ERROR] %s' % e)
		sys.exit(POSTPROCESS_ERROR)

	strRenameMask = ""
	show_type = 0
	for regex in show_se_parsers:
		if regex.search(originalname) != None:
			strRenameMask = os.environ.get('NZBPO_SEASONEPISODEMASK', '')
			show_type = 1
			break
	if show_type == 0:
		for regex in show_date_parsers:
			if regex.search(originalname) != None:
				strRenameMask = os.environ.get('NZBPO_BROADCASTDATEMASK', '')
				show_type = 2
				break

	# Is this a show of interest?
	if show_type == 0:
		# No, we're done here
		if True:
			print "Not interested in \"%s\"" % originalname
		sys.exit(POSTPROCESS_NONE)
	elif DEBUG:
		if show_type == 1:
			print "Series/Episode finish: \"%s\"" % originalname
		elif show_type == 2:
			print "Broadcast Date finish: \"%s\"" % originalname
		print "File mask: \"%s\"" % strRenameMask
	# We have a show of interest, but do we know what to do with it?
	if DEBUG:
		print "rename mask = \"%s\"" % strRenameMask
	if (len(strRenameMask) > 0):
		filelist=glob.glob("%s/%s.*" % (finaldirectory, originalname))
		if len(filelist) > 0:
			for file in filelist:
				match = extparser.search(file);
				if( match != None):
					success = (process(file, strRenameMask)==0)
					if(not success):
						print  "[ERROR] Failed processing '%s'." % (file)
						break
		else:
			success = True
	else:
		if show_type == 1:
			mask_type = "Season Episode"
		else:
			mask_type = "Broadcast Date"
		print "[ERROR] The file renaming mask for %s shows was not specified." % mask_type

	if(success):
		sys.exit(POSTPROCESS_SUCCESS)
	else:
		print "[ERROR] Uh-oh..."
		sys.exit(POSTPROCESS_ERROR)


